 /*
 *  linux/arch/arm/mach-pxa/colibri.c
 *
 *  Support for Toradex Colibri module
 *
 *  Orignal Author: Detlef Vollmann, vollmann engineering
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/irq.h>
#include <linux/m41t00.h>

#include <asm/setup.h>
#include <asm/memory.h>
#include <asm/mach-types.h>
#include <asm/hardware.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/arch/pxafb.h>
#include <asm/arch/mmc.h>
#include <asm/arch/udc.h>
#include <asm/arch/ohci.h>

#include <asm/arch/colibri.h>

#include "generic.h"

#ifdef CONFIG_DM9000
#include <linux/dm9000.h>
#endif

#ifdef CONFIG_I2C_PXA
#include <linux/i2c.h>
#include <asm/arch/i2c.h>
#endif

#if defined(CONFIG_VIDEO_PXA27x) || defined(CONFIG_VIDEO_PXA27x_MODULE)
#include <media/soc_camera.h>
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
#include <media/soc_ov7620.h>
#endif
#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
#include <media/soc_ov7720.h>
#endif
#include <asm/arch/camera.h>
#endif

static void __init colibri_init_irq(void)
{
	pxa_init_irq();

	/* setup extra colibri irqs */
    /* set_irq_type has to be called before an irq can be requested */
#ifdef CONFIG_DM9000
    set_irq_type(COLIBRI_ETH_IRQ, IRQT_RISING);
#endif
}

#ifdef CONFIG_DM9000
static struct resource dm9000_resources[] = {
    [0] = {
        .start  = COLIBRI_ETH_BASE,
        .end    = (COLIBRI_ETH_BASE + COLIBRI_ETH_SIZE),
        .flags  = IORESOURCE_MEM,
    },
    [1] = {
        .start  = (COLIBRI_ETH_IRQ),
        .end    = (COLIBRI_ETH_IRQ),
        .flags  = IORESOURCE_IRQ,
    },
};

static struct dm9000_plat_data dm9000_setup = {
    .flags          = DM9000_PLATF_32BITONLY
};

static struct platform_device dm9000_device = {
    .name           = "dm9000",
    .id             = 0,
    .num_resources  = ARRAY_SIZE(dm9000_resources),
    .resource       = dm9000_resources,
    .dev            = {
        .platform_data = &dm9000_setup,
    }
};
#endif

#ifdef CONFIG_SND_PXA2XX_AC97
static struct platform_device ac97_device = {
    .name           = "pxa2xx-ac97",
    .id             = -1,
};
#endif

#ifdef CONFIG_SENSORS_M41T00
static struct m41t00_platform_data rtc_setup = {
    .type = M41T00_TYPE_M41T00,
    .i2c_addr = M41T00_I2C_ADDR
};
static struct platform_device m41t00_dev = {
    .name = M41T00_DRV_NAME,
    .id = 0,
    .dev = {
        .platform_data = &rtc_setup
    }
};
#endif

#if defined(CONFIG_VIDEO_PXA27x) || defined(CONFIG_VIDEO_PXA27x_MODULE)
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
static struct ov7620_platform_data ov7620_data = {
  .iclink = {
    .bus_id = 0, /* Must match with camera ID in pxa27x.c */
    .gpio = -1,  /* no gpio */
  },
  .i2c_addr = OV7620_I2C_ADDR,
};
static struct platform_device ov7620_device = {
    .name = OV7620_DRV_NAME,
    .id = 0,
    .dev = {
        .platform_data = &ov7620_data,
    }
};
#endif
#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
static struct ov7720_platform_data ov7720_data = {
  .iclink = {
    .bus_id = 0, /* Must match with camera ID in pxa27x.c */
  },
  .i2c_addr = OV7720_I2C_ADDR,
};
static struct platform_device ov7720_device = {
    .name = OV7720_DRV_NAME,
    .id = 0,
    .dev = {
        .platform_data = &ov7720_data,
    }
};
#endif
#endif

static struct platform_device *devices[] __initdata = {
#ifdef CONFIG_DM9000
    &dm9000_device,
#endif
#ifdef CONFIG_SND_PXA2XX_AC97
    &ac97_device,
#endif
#ifdef CONFIG_SENSORS_M41T00
    &m41t00_dev,
#endif
#if defined(CONFIG_VIDEO_PXA27x) || defined(CONFIG_VIDEO_PXA27x_MODULE)
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
    &ov7620_device,
#endif
#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
    &ov7720_device,
#endif
#endif
};

/*
 * Video
 */
#ifdef CONFIG_FB_PXA
static void colibri_lcd_power(int on)
{
    pr_debug("%s: on = %d\n", __FUNCTION__, on);
    if (on) {
        GPDR(COLIBRI_LCD_BLO_GPIO) |= GPIO_bit(COLIBRI_LCD_BLO_GPIO);
        GPSR(COLIBRI_LCD_BLO_GPIO) |= GPIO_bit(COLIBRI_LCD_BLO_GPIO);

        CKEN |= CKEN0_PWM0;
    } else {
        GPCR(COLIBRI_LCD_BLO_GPIO) |= GPIO_bit(COLIBRI_LCD_BLO_GPIO);
        CKEN &= ~CKEN0_PWM0;
    }
}

static struct pxafb_mode_info colfb_mode = {
    .pixclock = LCD_PIXCLOCK,
    .xres = LCD_XRES,
    .yres = LCD_YRES,
    .bpp = COLIBRI_INIT_DEPTH,
    .lcd_bpp = COLIBRI_LCD_LINES,
    .hsync_len = LCD_HORIZONTAL_SYNC_PULSE_WIDTH,
    .left_margin = LCD_BEGIN_OF_LINE_WAIT_COUNT,
    .right_margin = LCD_END_OF_LINE_WAIT_COUNT,
    .vsync_len = LCD_VERTICAL_SYNC_PULSE_WIDTH,
    .upper_margin = LCD_BEGIN_FRAME_WAIT_COUNT,
    .lower_margin = LCD_END_OF_FRAME_WAIT_COUNT,
    .sync = LCD_SYNC,
    .cmap_greyscale = 0
};
static struct pxafb_mach_info colfb = {
    .modes = &colfb_mode,
    .num_modes = 1,
    .cmap_inverse = 0,
    .cmap_static = 0,
    .lccr0 = LCD_LCCR0,
    .lccr3 = LCD_LCCR3, /* pixel data format 4 */
    .lccr4 = 0x800a0000,
    .pxafb_backlight_power = colibri_lcd_power
};
#endif

/*
 * MMC/SD Device
 *
 */
#ifdef CONFIG_MMC_PXA
static int colibri_mci_init(struct device *dev, irq_handler_t detect_int, void *data)
{
    int err;

    printk("Colibri MMC/SD setup ");
	/* setup GPIO for PXA25x MMC controller	*/
	pxa_gpio_mode(GPIO_MMCCLK_AF);
	pxa_gpio_mode(GPIO_MMCCMD_AF);
	pxa_gpio_mode(GPIO_MMCDAT0_AF);
	pxa_gpio_mode(GPIO_MMCDAT1_AF);
	pxa_gpio_mode(GPIO_MMCDAT2_AF);
	pxa_gpio_mode(GPIO_MMCDAT3_AF);
	//pxa_gpio_mode(GPIO_MMCCS0_AF);

	err = request_irq(COLIBRI_MMC_CD_IRQ, detect_int, SA_INTERRUPT,
			     "MMC card detect", data);
	if (err) {
		printk(KERN_ERR "colibri_mci_init: MMC/SD: can't request MMC card detect IRQ\n");
		return -1;
	}

	set_irq_type(COLIBRI_MMC_CD_IRQ, IRQT_BOTHEDGE);

    printk("done.\n");
	return 0;
}

static struct pxamci_platform_data colibri_mci_platform_data = {
	.ocr_mask	= MMC_VDD_32_33|MMC_VDD_33_34,
	.init 		= colibri_mci_init,
};
#endif

/*
 * USB Device
 *
 */
#ifdef CONFIG_USB_GADGET
/*
 * udc_connected - is the USB cable connected
 *
 * Return non-zeron if cable is connected.
 */
static int udc_connected(void)
{
    int rc = 0;
# if (COLIBRI_USBCABLE_ACTIVE_HIGH)
    if ((GPLR(COLIBRI_USBCABLE_GPIO) & GPIO_bit(COLIBRI_USBCABLE_GPIO)) != 0)
        rc = 1;
# else
    if ((GPLR(COLIBRI_USBCABLE_GPIO) & GPIO_bit(COLIBRI_USBCABLE_GPIO)) == 0)
        rc = 1;
# endif

    pr_debug("GPIO: %08x, GPLR1: %08x, connected: %d", GPLR(COLIBRI_USBCABLE_GPIO) & GPIO_bit(COLIBRI_USBCABLE_GPIO), GPLR1, rc);

    return rc;
}

/*
 * udc_cmd_connect - en-/dis-able pullup resistor
 *
 * Turn on/off the USB connection by en-/dis-abling the pullup resistor.
 * For the Colibri eval board, we do nothing right now
 */
void udc_cmd_connect (int cmd)
{
    switch (cmd)
    {
    case PXA2XX_UDC_CMD_DISCONNECT:
        pr_debug("USB cmd disconnect\n");
        break;

    case PXA2XX_UDC_CMD_CONNECT:
        pr_debug("USB cmd connect\n");
        break;

    default:
        printk(KERN_ERR "%s: wrong command %d\n", __FUNCTION__, cmd);
    }

}

static struct pxa2xx_udc_mach_info colibri_udc_platform_data = {
	.udc_is_connected   = udc_connected,
	.udc_command 		= udc_cmd_connect,
};
#endif

#ifdef CONFIG_USB
static int colibri_ohci_init(struct device *dev)
{
	/* setup Port1 GPIO pin. */
	pxa_gpio_mode( 88 | GPIO_ALT_FN_1_IN);	/* USBHPWR1 */
	pxa_gpio_mode( 89 | GPIO_ALT_FN_2_OUT);	/* USBHPEN1 */

#ifndef CONFIG_USB_GADGET
    /* setup Port2 GPIO pin alternate functions */
    pxa_gpio_mode(119 | GPIO_ALT_FN_1_IN);	/* USBHPWR2 */
    pxa_gpio_mode(120 | GPIO_ALT_FN_2_OUT);	/* USBHPEN2 */
    UP2OCR = UP2OCR_HXOE | UP2OCR_HXS; /* use port 2 as host */
    UP2OCR |= UP2OCR_DPPDE | UP2OCR_DMPDE; /* enable D +/- pull-down */
#endif

	/* Set the Power Control Polarity Low and Power Sense
	   Polarity Low to active low. */
	UHCHR = (UHCHR | UHCHR_PCPL | UHCHR_PSPL) &
		~(UHCHR_SSEP1 | UHCHR_SSEP2 | UHCHR_SSEP3 | UHCHR_SSE);

	return 0;
}

static struct pxaohci_platform_data colibri_ohci_platform_data = {
	.port_mode	= PMM_PERPORT_MODE,
	.init		= colibri_ohci_init,
};
#endif

#if defined(CONFIG_VIDEO_PXA27x) || defined(CONFIG_VIDEO_PXA27x_MODULE)
static int colibri_pxacamera_init(struct device *dev)
{
  pxa_gpio_mode(GPIO_CIF_DD0);
  pxa_gpio_mode(GPIO_CIF_DD1);
  pxa_gpio_mode(GPIO_CIF_DD2);
  pxa_gpio_mode(GPIO_CIF_DD3);
  pxa_gpio_mode(GPIO_CIF_DD4);
  pxa_gpio_mode(GPIO_CIF_DD5);
  pxa_gpio_mode(GPIO_CIF_DD6);
  pxa_gpio_mode(GPIO_CIF_DD7);
#ifdef GPIO_CIF_DD8
  pxa_gpio_mode(GPIO_CIF_DD8);
#else
  #warning No pin for CIF_DD<8>
#endif
#ifdef GPIO_CIF_DD9
  pxa_gpio_mode(GPIO_CIF_DD9);
#else
  #warning No pin for CIF_DD<9>
#endif
#ifdef GPIO_CIF_MCLK
  pxa_gpio_mode(GPIO_CIF_MCLK);
#else
  #warning No pin for CIF_MCLK
#endif
  pxa_gpio_mode(GPIO_CIF_PCLK);
  pxa_gpio_mode(GPIO_CIF_LV);
  pxa_gpio_mode(GPIO_CIF_FV);

/* 	pxa_gpio_mode(  81 | GPIO_ALT_FN_2_IN); /\* CIF_DD[0] *\/ */
/* 	pxa_gpio_mode( 105 | GPIO_ALT_FN_1_IN); /\* CIF_DD[1] *\/ */
/* 	pxa_gpio_mode( 116 | GPIO_ALT_FN_1_IN); /\* CIF_DD[2] *\/ */
/* 	pxa_gpio_mode( 103 | GPIO_ALT_FN_1_IN); /\* CIF_DD[3] *\/ */
/* 	pxa_gpio_mode(  90 | GPIO_ALT_FN_3_IN); /\* CIF_DD[4] *\/ */
/* 	pxa_gpio_mode(  94 | GPIO_ALT_FN_2_IN); /\* CIF_DD[5] *\/ */
/* 	pxa_gpio_mode(  93 | GPIO_ALT_FN_2_IN); /\* CIF_DD[6] *\/ */
/* 	pxa_gpio_mode( 108 | GPIO_ALT_FN_1_IN); /\* CIF_DD[7] *\/ */
/* 	pxa_gpio_mode(  53 | GPIO_ALT_FN_2_OUT); /\* CIF_MCLK *\/ */
/* 	pxa_gpio_mode(  54 | GPIO_ALT_FN_3_IN);  /\* CIF_PCLK *\/ */
/* 	pxa_gpio_mode(  85 | GPIO_ALT_FN_3_IN);  /\* CIF_LV *\/ */
/* 	pxa_gpio_mode(  84 | GPIO_ALT_FN_3_IN);  /\* CIF_FV *\/ */

	return 0;
}

/*
 * CICR4:
 *      MASTER:		PXA receives sync signals from sensor
 *      PCLK_EN:	Pixel clock is supplied by the sensor
 *	~MCLK_EN:	No master clock output
 *	~PCP:		Data sampled on the rising edge of pixel clock
 *	~HSP:		Active high horizontal sync
 *	~VSP:		Active high vertical sync
 */
static struct pxacamera_platform_data colibri_pxacamera_platform_data = {
	.init	= colibri_pxacamera_init,
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
	.flags  = PXA_CAMERA_MASTER |
	          PXA_CAMERA_DATAWIDTH_8 |
	          PXA_CAMERA_PCLK_EN,
#endif
#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
	.flags  = PXA_CAMERA_MASTER |
	          PXA_CAMERA_DATAWIDTH_8 |
	          PXA_CAMERA_PCLK_EN |
	          PXA_CAMERA_MCLK_EN,
#endif
	.mclk_10khz = 2600,
};
#endif /* CONFIG_VIDEO_PXA27x ||CONFIG_VIDEO_PXA27x_MODULE */

static void __init colibri_init(void)
{
//#ifdef CONFIG_SND_PXA2XX_AC97
//    /* reset UCB1400 */
//    GPSR2 &= ~(1u << 31);
//    pxa_gpio_mode(GPIO_AC97_RESET | GPIO_OUT);
//    udelay(12);
//#endif
#ifdef CONFIG_MMC_PXA
    pxa_set_mci_info(&colibri_mci_platform_data);
#endif
#ifdef CONFIG_FB_PXA
    set_pxa_fb_info(&colfb);
#endif
    platform_add_devices(devices, ARRAY_SIZE(devices));
#ifdef CONFIG_USB_GADGET
    pxa_set_udc_info(&colibri_udc_platform_data);
#endif
#ifdef CONFIG_USB
    pxa_set_ohci_info(&colibri_ohci_platform_data);
#endif
#ifdef CONFIG_I2C_PXA
    pxa_set_i2c_info(NULL);
#endif
#if defined(CONFIG_VIDEO_PXA27x) || defined(CONFIG_VIDEO_PXA27x_MODULE)
    pxa_set_camera_info(&colibri_pxacamera_platform_data);
#endif
}

static void __init
fixup_colibri(struct machine_desc *desc, struct tag *tags,
		char **cmdline, struct meminfo *mi)
{
	SET_BANK (0, 0xa0000000, 64*1024*1024);
	mi->nr_banks      = 1;
}

#ifdef CONFIG_DM9000
static struct map_desc colibri_io_desc[] __initdata = {
 /* virtual         physical            length              type */
/*{ COLIBRI_FLASH,  COLIBRI_FLASH_PHYS, COLIBRI_FLASH_SIZE, MT_DEVICE },*/
  {
      .virtual  = COLIBRI_ETH_BASE,
      .pfn      = __phys_to_pfn(COLIBRI_ETH_PHYS),
      .length   = COLIBRI_ETH_SIZE,
      .type     = MT_DEVICE
  }
};
#endif

static void __init colibri_map_io(void)
{
	pxa_map_io();
#ifdef CONFIG_DM9000
	iotable_init(colibri_io_desc, ARRAY_SIZE(colibri_io_desc));
#endif

#ifdef CONFIG_PXA_FFUART
	/* enabling FFUART */
	CKEN |= CKEN6_FFUART;
	pxa_gpio_mode(GPIO_FFUART_TXD);
	pxa_gpio_mode(GPIO_FFUART_RXD);
	pxa_gpio_mode(GPIO_FFUART_RTS);
	pxa_gpio_mode(GPIO_FFUART_CTS);
	pxa_gpio_mode(GPIO_FFUART_DSR);
	pxa_gpio_mode(GPIO_FFUART_DTR);
	pxa_gpio_mode(GPIO_FFUART_DCD);
	pxa_gpio_mode(GPIO_FFUART_RI);
#endif

#ifdef CONFIG_PXA_BTUART
	/* enabling BTUART */
	CKEN |= CKEN7_BTUART;
	pxa_gpio_mode(GPIO_BTUART_TXD);
	pxa_gpio_mode(GPIO_BTUART_RXD);
	pxa_gpio_mode(GPIO_BTUART_RTS);
	pxa_gpio_mode(GPIO_BTUART_CTS);
#endif

#ifdef CONFIG_PXA_STUART
	/* enabling STUART */
	CKEN |= CKEN5_STUART;
	pxa_gpio_mode(GPIO_STUART_TXD);
	pxa_gpio_mode(GPIO_STUART_RXD);
#endif

#ifdef CONFIG_DM9000
	/* This is for the Davicom chip select */
	pxa_gpio_mode(GPIO78_nCS_2_MD);
#endif

	/* setup sleep mode values */
	PWER  = 0x00000002;
	PFER  = 0x00000000;
	PRER  = 0x00000002;
	PGSR0 = 0x00008000;
	PGSR1 = 0x003F0202;
	PGSR2 = 0x0001C000;
	PCFR |= PCFR_OPDE;
}

/*	MAINTAINER("dv@vollmann.ch, vollmann engineering gmbh") */
MACHINE_START(COLIBRI, "Toradex Colibri Module")
//	.phys_ram     = 0xa0000000,
	.phys_io      = 0x40000000,
	.io_pg_offst  = (io_p2v(0x40000000) >> 18) & 0xfffc,
	.boot_params  = 0xa0000100,
	.fixup        = fixup_colibri,
	.map_io       = colibri_map_io,
	.init_irq     = colibri_init_irq,
	.timer        = &pxa_timer,
	.init_machine = colibri_init,
MACHINE_END
