/*
 * Omnivision OV7620 Driver with soc_camera bus
 *
 * Copyright (C) 2008, Florian Pantaleao <fpantaleao@mobisensesystems.com>
 *
 * Inspired from Guennadi Liakhovetski mt9v022.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#define DEBUG

#include <linux/slab.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <linux/log2.h>

#define	OV7620_I2C_ADDR		0x21


static unsigned short ignore[] = { I2C_CLIENT_END };
static unsigned short normal_addr[] = {OV7620_I2C_ADDR, I2C_CLIENT_END };

static struct i2c_client_address_data addr_data = {
	.normal_i2c	= normal_addr,
	.probe		= ignore,
	.ignore		= ignore,
};

/* Forward declarations */
static struct i2c_driver ov7620_i2c_driver;

/* Manufacture ID constants */
#define OV7620_MIDH_VAL		0x7F
#define OV7620_MIDL_VAL		0xA2

/* OV7620 selected register addresses */
#define OV7620_AGC		0x0000
#define OV7620_BLUE		0x0001
#define OV7620_RED		0x0002
#define OV7620_SAT		0x0003
#define OV7620_BRT		0x0006
#define OV7620_SHP		0x0007
#define OV7620_ABLU		0x000C
#define OV7620_ARED		0x000D
#define OV7620_AEC		0x0010
#define OV7620_CLK		0x0011
#define OV7620_COMA		0x0012
#define OV7620_COMB		0x0013
#define OV7620_COMC		0x0014
#define OV7620_COMD		0x0015
#define OV7620_FD		0x0016
#define OV7620_HSTART		0x0017
#define OV7620_HSTOP		0x0018
#define OV7620_VSTRT		0x0019
#define OV7620_VSTOP		0x001A
#define OV7620_PSHFT		0x001B
#define OV7620_MIDH		0x001C		// Manufacture ID High byte
#define OV7620_MIDL		0x001D		// Manufacture ID Low byte
#define OV7620_COME		0x0020
#define OV7620_Y		0x0021
#define OV7620_U		0x0022
#define OV7620_CC		0x0023
#define OV7620_AEW		0x0024
#define OV7620_AEB		0x0025
#define OV7620_COMF		0x0026
#define OV7620_COMG		0x0027
#define OV7620_COMH		0x0028
#define OV7620_COMI		0x0029
#define OV7620_EHSH		0x002A
#define OV7620_EHSL		0x002B
#define OV7620_EXBK		0x002C
#define OV7620_COMJ		0x002D
#define OV7620_V		0x002E
#define OV7620_SPCA		0x0060
#define OV7620_SPCB		0x0061
#define OV7620_RGM		0x0062
#define OV7620_YGM		0x0064
#define OV7620_SPCC		0x0065
#define OV7620_AWBC		0x0066
#define OV7620_YUV		0x0067
#define OV7620_SPCD		0x0068
#define OV7620_EDGE		0x0069
#define OV7620_VEG		0x006A
#define OV7620_EOC		0x006F
#define OV7620_COMK		0x0070
#define OV7620_COML		0x0071
#define OV7620_HSDY		0x0072
#define OV7620_HEDY		0x0073
#define OV7620_COMM		0x0074
#define OV7620_COMN		0x0075
#define OV7620_COMO		0x0076
#define OV7620_AVG		0x007C

// End of OV7620 register
#define OV7620_REGEND		( OV7620_AVG + 1 )

/* Progressive scan, master, defaults */
/* #define MT9V022_CHIP_CONTROL_DEFAULT	0x188 */

struct ov7620 {
	struct i2c_client *client;
};

struct ov7620 ov7620_ctx;

/* static int reg_read(struct soc_camera_device *icd, const u8 reg) */
/* { */
/* 	struct ov7620 *ov7620 = container_of(icd, struct ov7620, icd); */
/* 	return i2c_smbus_read_byte_data(ov7620->client, reg); */
/* } */

static int reg_read(struct i2c_client *client, const u8 reg)
{
	char buf[1];
	buf[0] = reg;
        if (i2c_master_send(client, buf, 1) != 1) {
                printk("i2c_master_send failed.\n");
                return -1;
        }

        if (i2c_master_recv(client, buf, 1) != 1) {
                printk("i2c_master_recv failed.");
                return -1;
        }
        return buf[0];
}

/* static int reg_write(struct soc_camera_device *icd, const u8 reg, */
/* 		     const u8 data) */
/* { */
/* 	struct ov7620 *ov7620 = container_of(icd, struct ov7620, icd); */
/* 	return i2c_smbus_write_byte_data(ov7620->client, reg, data); */
/* } */

/* static int reg_set(struct soc_camera_device *icd, const u8 reg, */
/* 		   const u8 data) */
/* { */
/* 	int ret; */

/* 	ret = reg_read(icd, reg); */
/* 	if (ret < 0) */
/* 		return ret; */
/* 	return reg_write(icd, reg, ret | data); */
/* } */

/* static int reg_clear(struct soc_camera_device *icd, const u8 reg, */
/* 		     const u8 data) */
/* { */
/* 	int ret; */

/* 	ret = reg_read(icd, reg); */
/* 	if (ret < 0) */
/* 		return ret; */
/* 	return reg_write(icd, reg, ret & ~data); */
/* } */


/* 	/\* Read out the chip manufacture ID registers *\/ */
/* 	data_h = reg_read(icd, OV7620_MIDH); */
/* 	printk("after reg_read MIDH 0x%x\n", data_h); */
/* 	data_l = reg_read(icd, OV7620_MIDL); */
/* 	printk("after reg_read MIDL 0x%x\n", data_l); */
/* 	if (data_h != OV7620_MIDH_VAL || data_l != OV7620_MIDL_VAL) { */
/* 		ret = -ENODEV; */
/* 		dev_warn(&icd->dev, "No OV7620 detected, MIDH=0x%x, MIDL=0x%x\n", */
/* 			 data_h, data_l); */
/* 		goto ei2c; */
/* 	} */



/*
 *****************************************************************************
 *
 *	I2C Driver Interface
 *
 *****************************************************************************
 */
static int ov7620_detect_client (struct i2c_adapter *adap, int addr, int kind)
{
	struct i2c_client *client;
	int rc;

	printk("ov7620_detect_client addr=0x%x\n", addr);

	if (!i2c_check_functionality(adap, I2C_FUNC_I2C
			| I2C_FUNC_SMBUS_BYTE_DATA))
		return -EIO;

	printk("i2c_check_functionality OK\n");

	client = kzalloc(sizeof(struct i2c_client), GFP_KERNEL);
	if (!client)
		return -ENOMEM;

	strlcpy(client->name, "OV7620", I2C_NAME_SIZE);
	client->addr = addr;
	client->adapter = adap;
	client->driver = &ov7620_i2c_driver;

	if ((rc = i2c_attach_client(client)))
		goto attach_err;

	printk("i2c_attach_client OK\n");

	ov7620_ctx.client = client;
	i2c_set_clientdata(client, &ov7620_ctx);

	printk("i2c_set_clientdata OK\n");

	mdelay(100);
	int val;
	val = reg_read(client, 0x1C);
	printk("reg_read returned %x\n", val);

	return 0;

attach_err:
	kfree(client);
	return rc;
}

static int ov7620_attach(struct i2c_adapter *adap)
{
	return i2c_probe(adap, &addr_data, ov7620_detect_client);
}

static int ov7620_detach(struct i2c_client *client)
{
	int rc;

	dev_info(&client->dev, "ov7620_detach\n");

	rc = i2c_detach_client(client);
	if (rc == 0) {
		kfree(client);
	}

	return rc;
}

static struct i2c_driver ov7620_i2c_driver = {
	.driver = {
		.name = "ov7620",
	},
	.id 		= I2C_DRIVERID_OV7620,
	.attach_adapter = ov7620_attach,
	.detach_client	= ov7620_detach,
};

static int __init ov7620_mod_init(void)
{
	printk("   >>>   ov7620_mod_init\n");

	int ret;
	ret = i2c_add_driver(&ov7620_i2c_driver);
	printk("ov7620_mod_init ret=%d\n", ret);

	return ret;
}

static void __exit ov7620_mod_exit(void)
{
	printk("   >>>   ov7620_mod_exit\n");
	i2c_del_driver(&ov7620_i2c_driver);
}

module_init(ov7620_mod_init);
module_exit(ov7620_mod_exit);

MODULE_DESCRIPTION("I2C driver for Omnivision OV7620 Camera");
MODULE_AUTHOR("Florian Pantaleao <fpantaleao@mobisensesystems.com>");
MODULE_LICENSE("GPL");
