/*
 * linux26/drivers/pcmcia/pxa2xx_colibri.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Copyright (c) 2002 Accelent Systems, Inc. All Rights Reserved
 * 
 * Colibri compact flash support, based on IDP board support
 * maybe it would be better to base it on Mainstone support...
 *
 * Copyright (c) 2005 Toradex, Detlef Vollmann
 *
 */

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/irq.h>
#include <linux/platform_device.h>

#include <pcmcia/ss.h>

#include <asm/delay.h>
#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/arch/colibri.h>

#include "soc_common.h"

//#define DEBUG_COL_PCMCIA

#ifdef DEBUG_COL_PCMCIA
#define MARK printk(KERN_INFO "%s: %d\n", __FUNCTION__, __LINE__);
#else
//#define MARK (0)
#endif
#ifdef PCMCIA_DEBUG
int pc_debug = PCMCIA_DEBUG;
#endif

static struct pcmcia_irqs irqs[] = {
    { 0, COLIBRI_PCMCIA0_CD_IRQ, "PCMCIA CD"},
};

static int colibri_pcmcia_init_dev(struct soc_pcmcia_socket *skt)
{
    printk("Colibri PCMCIA\n");
    /* set power GPIO; switch off */
    GPCR(GPIO_PCMCIA_POW_EN) |= GPIO_bit(GPIO_PCMCIA_POW_EN);
    GPDR(GPIO_PCMCIA_POW_EN) |= GPIO_bit(GPIO_PCMCIA_POW_EN);

    /* set PCMCIA AFs */
    GPSR(GPIO_PCMCIA_NPOE) |= GPIO_bit(GPIO_PCMCIA_NPOE);
    pxa_gpio_mode(GPIO_PCMCIA_NPOE | GPIO_PCMCIA_NPOE_AF);

    GPSR(GPIO_PCMCIA_NPIOR) |= GPIO_bit(GPIO_PCMCIA_NPIOR);
    pxa_gpio_mode(GPIO_PCMCIA_NPIOR | GPIO_PCMCIA_NPIOR_AF);

    GPSR(GPIO_PCMCIA_NPIOW) |= GPIO_bit(GPIO_PCMCIA_NPIOW);
    pxa_gpio_mode(GPIO_PCMCIA_NPIOW | GPIO_PCMCIA_NPIOW_AF);

    GPSR(GPIO_PCMCIA_NPCE1) |= GPIO_bit(GPIO_PCMCIA_NPCE1);
    pxa_gpio_mode(GPIO_PCMCIA_NPCE1 | GPIO_PCMCIA_NPCE1_AF);

    GPCR(GPIO_PCMCIA_NPCE2) |= GPIO_bit(GPIO_PCMCIA_NPCE2);
    pxa_gpio_mode(GPIO_PCMCIA_NPCE2 | GPIO_PCMCIA_NPCE2_AF);

    GPSR(GPIO_PCMCIA_NPREG) |= GPIO_bit(GPIO_PCMCIA_NPREG);
    pxa_gpio_mode(GPIO_PCMCIA_NPREG | GPIO_PCMCIA_NPREG_AF);

    pxa_gpio_mode(GPIO_PCMCIA_NPWAIT | GPIO_PCMCIA_NPWAIT_AF);

    pxa_gpio_mode(GPIO_PCMCIA_NPIOIS16 | GPIO_PCMCIA_NPIOIS16_AF);

    GPSR(GPIO_PCMCIA_PSKTSEL) |= GPIO_bit(GPIO_PCMCIA_PSKTSEL);
    pxa_gpio_mode(GPIO_PCMCIA_PSKTSEL | GPIO_PCMCIA_PSKTSEL_AF);

    /* set other PCMCIA GPIOs */
    GPCR(GPIO_PCMCIA0_RESET) |= GPIO_bit(GPIO_PCMCIA0_RESET);
    GPDR(GPIO_PCMCIA0_RESET) |= GPIO_bit(GPIO_PCMCIA0_RESET);

    GPDR(GPIO_PCMCIA0_BVD1) &= ~GPIO_bit(GPIO_PCMCIA0_BVD1);
    GPDR(GPIO_PCMCIA0_BVD2) &= ~GPIO_bit(GPIO_PCMCIA0_BVD2);

    /* switch power on */
    PCC_PWR_ON();

	/* reset the PCMCIA controller */
    GPSR(GPIO_PCMCIA0_RESET) |= GPIO_bit(GPIO_PCMCIA0_RESET);
	udelay(500);
	/* un-reset it again */
    GPCR(GPIO_PCMCIA0_RESET) |= GPIO_bit(GPIO_PCMCIA0_RESET);

    /* set interrupts */
	GPDR(GPIO_PCMCIA0_CD_IRQ) &= ~GPIO_bit(GPIO_PCMCIA0_CD_IRQ);
	set_irq_type(GPIO_PCMCIA0_CD_IRQ, COLIBRI_PCMCIA_CD_EDGE);

	GPDR(GPIO_PCMCIA0_RDY_IRQ) &= ~GPIO_bit(GPIO_PCMCIA0_RDY_IRQ);
	set_irq_type(GPIO_PCMCIA0_RDY_IRQ, COLIBRI_PCMCIA_RDY_EDGE);

    skt->irq = COLIBRI_PCMCIA0_RDY_IRQ;
    return soc_pcmcia_request_irqs(skt, irqs, ARRAY_SIZE(irqs));
}

static void colibri_pcmcia_shutdown(struct soc_pcmcia_socket *skt)
{
	soc_pcmcia_free_irqs(skt, irqs, ARRAY_SIZE(irqs));

    /* switch power off */
    PCC_PWR_OFF();
}

static void colibri_pcmcia_socket_state(struct soc_pcmcia_socket *skt,
                                        struct pcmcia_state *state)
{
	//memset(state, 0, sizeof(*state));

    /* these are gpios */
	state->detect = (PCC_DETECT) ? 1 : 0;    /* active high??? */
	state->ready =  (PCC_READY) ? 1 : 0;

/*
    printk(KERN_INFO "CF status: detect: %u, ready: %u\n",
           GPLR(84) & GPIO_bit(84),
           GPLR(1) & GPIO_bit(1));
*/

	state->bvd1   = PCC_BVD1() ? 1 : 0;
	state->bvd2   = PCC_BVD2() ? 1 : 0;
	state->wrprot = 0; /* r/w all the time */
	state->vs_3v  = PCC_VS3V() ? 1 : 0;
	state->vs_Xv  = PCC_VS5V() ? 1 : 0;
}

static int colibri_pcmcia_configure_socket(struct soc_pcmcia_socket *skt,
                                           socket_state_t const *state)
{
    unsigned long flags;

    local_irq_save(flags);
	/* configure Vcc and Vpp */
    if (state->Vcc == 0 && state->Vpp == 0)
    {
        PCC_PWR_OFF();
    }
    else if (state->Vcc == 33 && state->Vpp < 50)
    {
        PCC_PWR_ON();
    }
    else
    {
        printk(KERN_ERR "%s(): unsupported Vcc %u Vpp %u combination\n",
               __FUNCTION__, state->Vcc, state->Vpp);
        return -1;
    }

    /* reset PCMCIA if requested */
    if (state->flags & SS_RESET)
    {
        GPSR(GPIO_PCMCIA0_RESET) |= GPIO_bit(GPIO_PCMCIA0_RESET);
    }

    local_irq_restore(flags);
    udelay(200);

	return 0;
}

static void colibri_pcmcia_socket_init(struct soc_pcmcia_socket *skt)
{
}

static void colibri_pcmcia_socket_suspend(struct soc_pcmcia_socket *skt)
{
}

struct pcmcia_low_level colibri_pcmcia_ops = { 
    .owner             = THIS_MODULE,
    .hw_init           = colibri_pcmcia_init_dev,
    .hw_shutdown       = colibri_pcmcia_shutdown,
    .socket_state      = colibri_pcmcia_socket_state,
    .configure_socket  = colibri_pcmcia_configure_socket,
    .socket_init       = colibri_pcmcia_socket_init,
    .socket_suspend    = colibri_pcmcia_socket_suspend,
    .first             = 0,
    .nr                = 1
};

static struct platform_device *colibri_pcmcia_device;

static int __init colibri_pcmcia_init(void)
{
    int ret;

    colibri_pcmcia_device = kmalloc(sizeof(*colibri_pcmcia_device), GFP_KERNEL);
    if (!colibri_pcmcia_device)
        return -ENOMEM;
    memset(colibri_pcmcia_device, 0, sizeof(*colibri_pcmcia_device));
    colibri_pcmcia_device->name = "pxa2xx-pcmcia";
    colibri_pcmcia_device->dev.platform_data = &colibri_pcmcia_ops;

    ret = platform_device_register(colibri_pcmcia_device);
    if (ret)
        kfree(colibri_pcmcia_device);

    return ret;
}

static void __exit colibri_pcmcia_exit(void)
{
    /* Are there still references to colibri_pcmcia_device?
     * I don't know, so I'd better not free it.
     * Actually, I don't really care, as I don't really support
     * this driver as module anyway...
     */
    platform_device_unregister(colibri_pcmcia_device);
}

module_init(colibri_pcmcia_init);
module_exit(colibri_pcmcia_exit);

MODULE_DESCRIPTION("Colibri PXA270 CF Support");
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Detlef Vollmann <detlef.vollmann@toradex.com>");
