/*
 *  linux/include/asm-arm/arch-pxa/colibri.h
 *  customized version for H&F
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Copyright (c) 2006 Detlef Vollmann, <dv@vollmann.ch>
 *
 */

#include <asm/arch/pxa-regs.h>
#include <asm/arch/irqs.h>


/*
 * Note: include file for assembler and C
 */

#ifndef __ASSEMBLY__
#define GPIO_SET(g) (GPSR(g) |= GPIO_bit(g))
#define GPIO_CLEAR(g) (GPCR(g) |= GPIO_bit(g))
#endif

/*
 * Video settings
 */

#if defined CONFIG_COLIBRI_LCD_VGA
/* settings for analog VGA connection */

/* set to 1 if you want these settings to be active */
#define COLIBRI_LCD_SETTINGS        1

/* set to GPIO that switches on the backlight */
#define COLIBRI_LCD_BLO_GPIO        81

/* set to 1 if the LCD is an active panel (TFT) or for VGA */
#define COLIBRI_LCD_ACTIVE          1

/* set to 1 if the LCD panel is a dual-scan panel */
#define COLIBRI_LCD_DUAL            0

/* set to 1 if the LCD BIAS pin should be active low (OEP) */
#define COLIBRI_BIAS_ACTIVE_LOW     0

/* set to 1 if the data is sampled on the falling edge of the pixel clock PCP*/
#define COLIBRI_PIX_CLK_FALLING     0

/* set to number of LCD data lines (physical depth) */
#define COLIBRI_LCD_LINES       18

/* set to 3 if more than 16 bpp (PDFOR) */
#define COLIBRI_PIXEL_DATA_FORMAT   3

/* set to initial buffer depth */
#define COLIBRI_INIT_DEPTH      8


/* the following settings can also be set using fbset */
/* these are settings are for analog VGA */
#define LCD_PIXCLOCK                    40000
#define LCD_XRES                        640
#define LCD_YRES                        480
#define LCD_HORIZONTAL_SYNC_PULSE_WIDTH 64
#define LCD_VERTICAL_SYNC_PULSE_WIDTH   2
#define LCD_BEGIN_OF_LINE_WAIT_COUNT    96
#define LCD_BEGIN_FRAME_WAIT_COUNT          33
#define LCD_END_OF_LINE_WAIT_COUNT          48
#define LCD_END_OF_FRAME_WAIT_COUNT         10
#define LCD_SYNC                        0


/* VGA timings:
pixclk 0x1801050 Hz (25MHz) = 40000ps
640x480 18BPP
LCD_HORIZONTAL_SYNC_PULSE_WIDTH HSPW 0x40
LCD_BEGIN_OF_LINE_WAIT_COUNT BLW 0x60
LCD_END_OF_LINE_WAIT_COUNT ELW 0x30
LCD_VERTICAL_SYNC_PULSE_WIDTH VSW 0x2
LCD_BEGIN_FRAME_WAIT_COUNT BFW 0x21
LCD_END_OF_FRAME_WAIT_COUNT EFW 0xa
COLIBRI_PIX_CLK_FALLING PCP 0
FB_SYNC_HOR_HIGH_ACT_LOW HSP 1
FB_SYNC_VERT_HIGH_ACT_LOW VSP 1
COLIBRI_LCD_DUAL SDS 0
COLIBRI_LCD_ACTIVE PAS 1
COLIBRI_BIAS_ACTIVE_LOW OEP 0
*/

#elif defined CONFIG_COLIBRI_LCD_LB064V02
/* settings for Philips LB064V02-A1 */

/* set to 1 if you want these settings to be active */
#define COLIBRI_LCD_SETTINGS        1

/* set to GPIO that switches on the backlight */
#define COLIBRI_LCD_BLO_GPIO        81

/* set to 1 if the LCD is an active panel (TFT) or for VGA */
#define COLIBRI_LCD_ACTIVE          1

/* set to 1 if the LCD panel is a dual-scan panel */
#define COLIBRI_LCD_DUAL            0

/* set to 1 if the LCD BIAS pin should be active low (OEP) */
#define COLIBRI_BIAS_ACTIVE_LOW     0

/* set to 1 if the data is sampled on the falling edge of the pixel clock PCP*/
#define COLIBRI_PIX_CLK_FALLING     1

/* set to number of LCD data lines (physical depth) */
#define COLIBRI_LCD_LINES       16

/* set to 3 if more than 16 bpp (PDFOR) */
/* set to 0 if 16 bpp (PDFOR) */
#define COLIBRI_PIXEL_DATA_FORMAT   0

/* set to initial buffer depth */
#define COLIBRI_INIT_DEPTH      16


/* the following settings can also be set using fbset */
/* these settings are for LB064V02-A1 */
#define LCD_PIXCLOCK                    40000
#define LCD_XRES                        640
#define LCD_YRES                        480
#define LCD_HORIZONTAL_SYNC_PULSE_WIDTH	2
#define LCD_VERTICAL_SYNC_PULSE_WIDTH	45
#define LCD_BEGIN_OF_LINE_WAIT_COUNT	160
#define LCD_BEGIN_FRAME_WAIT_COUNT 	    0
#define LCD_END_OF_LINE_WAIT_COUNT	    2
#define LCD_END_OF_FRAME_WAIT_COUNT	    0
#define LCD_SYNC			(FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT)

/* Philips LB064V02-A1 timings:
pixclk 0x1801050 Hz (25MHz) = 40000ps
640x480 18BPP
LCD_HORIZONTAL_SYNC_PULSE_WIDTH HSPW 2
LCD_BEGIN_OF_LINE_WAIT_COUNT BLW 160
LCD_END_OF_LINE_WAIT_COUNT ELW 2
LCD_VERTICAL_SYNC_PULSE_WIDTH VSW 45
LCD_BEGIN_FRAME_WAIT_COUNT BFW 0
LCD_END_OF_FRAME_WAIT_COUNT EFW 0
COLIBRI_PIX_CLK_FALLING PCP 1
FB_SYNC_HOR_HIGH_ACT_LOW HSP 0
FB_SYNC_VERT_HIGH_ACT_LOW VSP 0
COLIBRI_LCD_DUAL SDS 0
COLIBRI_LCD_ACTIVE PAS 1
COLIBRI_BIAS_ACTIVE_LOW OEP 0
*/

#elif defined CONFIG_COLIBRI_LCD_LQ043T3DX02
/* settings for Sharp LQ043T3DX02 */

/* set to 1 if you want these settings to be active */
#define COLIBRI_LCD_SETTINGS        1

/* set to GPIO that switches on the backlight */
#define COLIBRI_LCD_BLO_GPIO        104

/* set to 1 if the LCD is an active panel (TFT) or for VGA */
#define COLIBRI_LCD_ACTIVE          1

/* set to 1 if the LCD panel is a dual-scan panel */
#define COLIBRI_LCD_DUAL            0

/* set to 1 if the LCD BIAS pin should be active low (OEP) */
/* Don't care: L_BIAS not connected */
#define COLIBRI_BIAS_ACTIVE_LOW     0

/* set to 1 if the data is sampled on the falling edge of the pixel clock PCP*/
#define COLIBRI_PIX_CLK_FALLING     1

/* set to number of LCD data lines (physical depth) */
#define COLIBRI_LCD_LINES       18

/* set to 3 if more than 16 bpp (PDFOR) */
/* set to 0 if 16 bpp (PDFOR) */
#define COLIBRI_PIXEL_DATA_FORMAT   3

/* set to initial buffer depth */
#define COLIBRI_INIT_DEPTH      18


/* the following settings can also be set using fbset */
/* these settings are for Sharp LQ043T3DX02 */
#define LCD_PIXCLOCK                    111111
#define LCD_XRES                        480
#define LCD_YRES                        272
#define LCD_HORIZONTAL_SYNC_PULSE_WIDTH	41
#define LCD_BEGIN_OF_LINE_WAIT_COUNT	2
#define LCD_END_OF_LINE_WAIT_COUNT	2
#define LCD_VERTICAL_SYNC_PULSE_WIDTH	10
#define LCD_BEGIN_FRAME_WAIT_COUNT 	2
#define LCD_END_OF_FRAME_WAIT_COUNT	2
#define LCD_SYNC			0

/* Sharp LQ043T3DX02 timings:
pixclk 9MHz => T=111111ps
480x272 18BPP
LCD_HORIZONTAL_SYNC_PULSE_WIDTH HSPW 41
LCD_BEGIN_OF_LINE_WAIT_COUNT BLW 2
LCD_END_OF_LINE_WAIT_COUNT ELW 2
LCD_VERTICAL_SYNC_PULSE_WIDTH VSW 10
LCD_BEGIN_FRAME_WAIT_COUNT BFW 2
LCD_END_OF_FRAME_WAIT_COUNT EFW 2
COLIBRI_PIX_CLK_FALLING PCP 1
FB_SYNC_HOR_HIGH_ACT_LOW HSP 1
FB_SYNC_VERT_HIGH_ACT_LOW VSP 1
COLIBRI_LCD_DUAL SDS 0
COLIBRI_LCD_ACTIVE PAS 1
COLIBRI_BIAS_ACTIVE_LOW OEP 0
*/
#endif /* LCD device */



/*
 * physical memory map
 */

#define COLIBRI_FLASH_PHYS		PXA_CS0_PHYS    /* 0x00000000 */
#define COLIBRI_FLASH_SIZE		(0x02000000)

#define COLIBRI_ETH_PHYS		PXA_CS2_PHYS    /* 0x08000000 */

/*
 * virtual memory map
 */

#define COLIBRI_FLASH           0x00000000
#define COLIBRI_ETH_BASE		0xfc000000
#define COLIBRI_ETH_SIZE        (1024*1024)

#define DM9000_BASE		        COLIBRI_ETH_BASE
#define DM9000_MMIO             1
#define DM9000_ADDRFROMTAG      1

/*
 * interrupts and GPIO
 */

/* FFUART */
#ifdef CONFIG_COLIBRI_ON_EVB
#define GPIO_FFUART_TXD GPIO39_FFTXD_MD
#define GPIO_FFUART_RXD GPIO34_FFRXD_MD
#define GPIO_FFUART_RTS GPIO27_FFRTS_MD
#define GPIO_FFUART_CTS GPIO100_FFCTS_MD
#define GPIO_FFUART_DSR GPIO33_FFDSR_MD
#define GPIO_FFUART_DTR GPIO40_FFDTR_MD
#define GPIO_FFUART_DCD GPIO10_FFDCD_MD
#define GPIO_FFUART_RI  GPIO38_FFRI_MD
#elif defined CONFIG_COLIBRI_ON_MBS270_V1
#define GPIO_FFUART_TXD GPIO39_FFTXD_MD
#define GPIO_FFUART_RXD GPIO34_FFRXD_MD
#define GPIO_FFUART_RTS GPIO27_FFRTS_MD
#define GPIO_FFUART_CTS GPIO100_FFCTS_MD
#define GPIO_FFUART_DSR GPIO33_FFDSR_MD
#define GPIO_FFUART_DTR GPIO40_FFDTR_MD
#define GPIO_FFUART_DCD GPIO36_FFDCD_MD
#define GPIO_FFUART_RI  GPIO38_FFRI_MD
#elif defined CONFIG_COLIBRI_ON_MBS270_V2_0
#define GPIO_FFUART_TXD GPIO37_FFTXD_MD
#define GPIO_FFUART_RXD GPIO34_FFRXD_MD
#define GPIO_FFUART_RTS GPIO27_FFRTS_MD
#define GPIO_FFUART_CTS GPIO100_FFCTS_MD
#define GPIO_FFUART_DSR GPIO33_FFDSR_MD
#define GPIO_FFUART_DTR GPIO40_FFDTR_MD
#define GPIO_FFUART_DCD GPIO10_FFDCD_MD
#define GPIO_FFUART_RI  GPIO38_FFRI_MD
#endif

/* BTUART */
#ifdef CONFIG_COLIBRI_ON_EVB
#define GPIO_BTUART_TXD GPIO43_BTTXD_MD
#define GPIO_BTUART_RXD GPIO42_BTRXD_MD
#define GPIO_BTUART_RTS GPIO45_BTRTS_MD
#define GPIO_BTUART_CTS GPIO44_BTCTS_MD
#elif defined CONFIG_COLIBRI_ON_MBS270_V1
#define GPIO_BTUART_TXD GPIO43_BTTXD_MD
#define GPIO_BTUART_RXD GPIO42_BTRXD_MD
#define GPIO_BTUART_RTS GPIO45_BTRTS_MD
#define GPIO_BTUART_CTS GPIO44_BTCTS_MD
#elif defined CONFIG_COLIBRI_ON_MBS270_V2_0
#define GPIO_BTUART_TXD GPIO43_BTTXD_MD
#define GPIO_BTUART_RXD GPIO42_BTRXD_MD
#define GPIO_BTUART_RTS GPIO45_BTRTS_MD
#define GPIO_BTUART_CTS GPIO44_BTCTS_MD
#endif

/* STUART */
#ifdef CONFIG_COLIBRI_ON_EVB
#define GPIO_STUART_TXD GPIO47_STTXD_MD
#define GPIO_STUART_RXD GPIO46_STRXD_MD
#elif defined CONFIG_COLIBRI_ON_MBS270_V1
#define GPIO_STUART_TXD GPIO47_STTXD_MD
#define GPIO_STUART_RXD GPIO46_STRXD_MD
#elif defined CONFIG_COLIBRI_ON_MBS270_V2_0
#define GPIO_STUART_TXD GPIO47_STTXD_MD
#define GPIO_STUART_RXD GPIO46_STRXD_MD
#endif

/* Ethernet */
#define GPIO_ETH_IRQ    114
#define COLIBRI_ETH_IRQ IRQ_GPIO(GPIO_ETH_IRQ)
#define DM9000_IRQ      COLIBRI_ETH_IRQ

/* Compact Flash/PCMCIA */
#define GPIO_PCMCIA0_CD_IRQ     84
#define COLIBRI_PCMCIA0_CD_IRQ  IRQ_GPIO(GPIO_PCMCIA0_CD_IRQ)
#define COLIBRI_PCMCIA_CD_EDGE  IRQT_BOTHEDGE
#define GPIO_PCMCIA0_RDY_IRQ    1
#define COLIBRI_PCMCIA0_RDY_IRQ IRQ_GPIO(GPIO_PCMCIA0_RDY_IRQ)
#define COLIBRI_PCMCIA_RDY_EDGE IRQT_FALLING

#ifdef CONFIG_COLIBRI_ON_EVB
#define GPIO_PCMCIA_POW_EN      107
#define GPIO_PCMCIA_NPOE        48
#define GPIO_PCMCIA_NPOE_AF     GPIO_ALT_FN_2_OUT
#define GPIO_PCMCIA_NPIOR       50
#define GPIO_PCMCIA_NPIOR_AF    GPIO_ALT_FN_2_OUT
#define GPIO_PCMCIA_NPIOW       51
#define GPIO_PCMCIA_NPIOW_AF    GPIO_ALT_FN_2_OUT
#define GPIO_PCMCIA_NPCE1       85
#define GPIO_PCMCIA_NPCE1_AF    GPIO_ALT_FN_1_OUT
#define GPIO_PCMCIA_NPCE2       54
#define GPIO_PCMCIA_NPCE2_AF    GPIO_ALT_FN_2_OUT
#define GPIO_PCMCIA_NPREG       55
#define GPIO_PCMCIA_NPREG_AF    GPIO_ALT_FN_2_OUT
#define GPIO_PCMCIA_NPWAIT      56
#define GPIO_PCMCIA_NPWAIT_AF   GPIO_ALT_FN_1_IN
#define GPIO_PCMCIA_NPIOIS16    57
#define GPIO_PCMCIA_NPIOIS16_AF GPIO_ALT_FN_1_IN
#define GPIO_PCMCIA_PSKTSEL     104
#define GPIO_PCMCIA_PSKTSEL_AF  GPIO_ALT_FN_1_OUT
#define GPIO_PCMCIA0_RESET      53
#define GPIO_PCMCIA0_BVD1       83
#define GPIO_PCMCIA0_BVD2       82
#endif

#define PCC_DETECT (GPLR(GPIO_PCMCIA0_CD_IRQ) & GPIO_bit(GPIO_PCMCIA0_CD_IRQ))
#define PCC_READY (GPLR(GPIO_PCMCIA0_RDY_IRQ) & GPIO_bit(GPIO_PCMCIA0_RDY_IRQ))
#define PCC_BVD1()  (GPLR(GPIO_PCMCIA0_BVD1) & GPIO_bit(GPIO_PCMCIA0_BVD1))
#define PCC_BVD2()  (GPLR(GPIO_PCMCIA0_BVD2) & GPIO_bit(GPIO_PCMCIA0_BVD2))
#define PCC_VS3V()  1       /* only 3.3V supported */
#define PCC_VS5V()  0       /* only 3.3V supported */
#define PCC_PWR_ON() (GPCR(GPIO_PCMCIA_POW_EN) |= GPIO_bit(GPIO_PCMCIA_POW_EN))
#define PCC_PWR_OFF() (GPSR(GPIO_PCMCIA_POW_EN) |= GPIO_bit(GPIO_PCMCIA_POW_EN))

#if defined(CONFIG_COLIBRI_ON_MBS270_V1) && defined(PCMCIA)
#error PCMCIA not available on MBS270-V1
#endif

#if defined(CONFIG_COLIBRI_ON_MBS270_V2_0) && defined(PCMCIA)
#error PCMCIA not available on MBS270-V2.0
#endif

/* MMC/SD */
#ifdef CONFIG_COLIBRI_ON_EVB
#define GPIO_MMC_CD_IRQ         0
#define GPIO_MMCCLK_AF          GPIO32_MMCCLK_MD
#define GPIO_MMCDAT0_AF          GPIO92_MMCDAT0_MD
#define GPIO_MMCDAT1_AF          GPIO109_MMCDAT1_MD
#define GPIO_MMCDAT2_AF          GPIO110_MMCDAT2_MD
#define GPIO_MMCDAT3_AF          GPIO111_MMCDAT3_MD
#define GPIO_MMCCMD_AF          GPIO112_MMCCMD_MD
//#define GPIO_MMCCS0_AF          GPIO110_MMCCS0_MD
#endif

#if defined(CONFIG_COLIBRI_ON_MBS270_V1) && defined(MMC)
#error MultiMedia Card not available on MBS270-V1
#endif

#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
#define GPIO_MMC_CD_IRQ         19
#define GPIO_MMCCLK_AF          GPIO32_MMCCLK_MD
#define GPIO_MMCDAT0_AF          GPIO92_MMCDAT0_MD
#define GPIO_MMCDAT1_AF          GPIO109_MMCDAT1_MD
#define GPIO_MMCDAT2_AF          GPIO110_MMCDAT2_MD
#define GPIO_MMCDAT3_AF          GPIO111_MMCDAT3_MD
#define GPIO_MMCCMD_AF          GPIO112_MMCCMD_MD
//#define GPIO_MMCCS0_AF          GPIO110_MMCCS0_MD
#endif

#define COLIBRI_MMC_CD_IRQ      IRQ_GPIO(GPIO_MMC_CD_IRQ)

/* TouchScreen and Sound */
#define GPIO_TOUCH_IRQ          113
#define COLIBRI_TOUCH_IRQ       IRQ_GPIO(GPIO_TOUCH_IRQ)

#define GPIO_AC97_RESET         95
#define GPIO_AC97_RST_AF        GPIO_ALT_FN_1_OUT
#define GPIO_AC97_SYSCLK        98
#define GPIO_AC97_SYSCLK_AF     GPIO_ALT_FN_1_OUT

/* USB device */
#define COLIBRI_USBCABLE_GPIO   41
#define COLIBRI_USBCABLE_ACTIVE_HIGH    1

/* LCD details */
#define LCD_LCCR0_BASE   (LCCR0_LDDALT | LCCR0_OUM | LCCR0_BM \
                          | LCCR0_QDM | LCCR0_EFM | LCCR0_IUM \
                          | LCCR0_SFM | LCCR0_LDM)

#define LCD_LCCR0        (LCD_LCCR0_BASE \
                          | ((COLIBRI_LCD_ACTIVE) << 7) \
                          | ((COLIBRI_LCD_DUAL) << 2))

#define LCD_LCCR3        (LCCR3_Acb(0xff) \
                          | ((COLIBRI_BIAS_ACTIVE_LOW) << 23) \
                          | ((COLIBRI_PIX_CLK_FALLING) << 22) \
                          | ((COLIBRI_PIXEL_DATA_FORMAT) << 30))

/* Unused GPIOs */
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
#define PXA_GPIO_UNUSED {1, 9, 10, 11, 12, 13, 14, 16, 17, 19, 20, 21, 22, \
	37, 41, 48, 50, 51, 55, 56, 57, 91, 96, 97, 99, 101, 102, 104, \
	106, 107, 115}
#endif

#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
#define PXA_GPIO_UNUSED {0, 1, 9, 11, 13, 14, 16, 19, 20, 21, 22, \
      36, 48, 50, 56, 57, 80, 82, 90, 91, 93, 94, 96, 97, 99, 101, 102,	\
        103, 104, 105, 108, 115, 116}
#endif

/* PWM */
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
#define PXA_PWM_GPIO {16, 17, 11, 12}
#define PXA_PWM_GPIO_ALT_FN \
 {GPIO_ALT_FN_2_OUT, GPIO_ALT_FN_2_OUT, GPIO_ALT_FN_2_OUT, GPIO_ALT_FN_2_OUT}
#endif

#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
#define PXA_PWM_GPIO {16, 115, 11, 80}
#define PXA_PWM_GPIO_ALT_FN \
 {GPIO_ALT_FN_2_OUT, GPIO_ALT_FN_3_OUT, GPIO_ALT_FN_2_OUT, GPIO_ALT_FN_3_OUT}
#endif

/* EXTRA TIMERS */
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
#define PXA_TIMER_GPIO {9, 10}
#define PXA_TIMER_GPIO_ALT_FN {GPIO_ALT_FN_3_OUT, GPIO_ALT_FN_3_OUT}
#endif

#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
#define PXA_TIMER_GPIO {11}
#define PXA_TIMER_GPIO_ALT_FN {GPIO_ALT_FN_1_OUT}
#endif

/* CAMERA INTERFACE */
#ifdef CONFIG_COLIBRI_ON_MBS270_V1
#define GPIO_CIF_DD0  ( 81 | GPIO_ALT_FN_2_IN)
#define GPIO_CIF_DD1  (105 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_DD2  (116 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_DD3  (103 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_DD4  ( 90 | GPIO_ALT_FN_3_IN)
#define GPIO_CIF_DD5  ( 94 | GPIO_ALT_FN_2_IN)
#define GPIO_CIF_DD6  ( 93 | GPIO_ALT_FN_2_IN)
#define GPIO_CIF_DD7  (108 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_MCLK ( 53 | GPIO_ALT_FN_2_OUT)
#define GPIO_CIF_PCLK ( 54 | GPIO_ALT_FN_3_IN)
#define GPIO_CIF_LV   ( 85 | GPIO_ALT_FN_3_IN)
#define GPIO_CIF_FV   ( 84 | GPIO_ALT_FN_3_IN)
#endif

#ifdef CONFIG_COLIBRI_ON_MBS270_V2_0
#define GPIO_CIF_DD0  ( 81 | GPIO_ALT_FN_2_IN)
#define GPIO_CIF_DD1  ( 55 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_DD2  ( 51 | GPIO_ALT_FN_1_IN)
/* #define GPIO_CIF_DD3  ( 50 | GPIO_ALT_FN_1_IN) */
#define GPIO_CIF_DD3  (103 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_DD4  ( 83 | GPIO_ALT_FN_3_IN)
#define GPIO_CIF_DD5  ( 48 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_DD6  ( 17 | GPIO_ALT_FN_2_IN)
#define GPIO_CIF_DD7  ( 12 | GPIO_ALT_FN_2_IN)
#define GPIO_CIF_DD8  (107 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_DD9  (106 | GPIO_ALT_FN_1_IN)
#define GPIO_CIF_MCLK ( 53 | GPIO_ALT_FN_2_OUT)
#define GPIO_CIF_PCLK ( 54 | GPIO_ALT_FN_3_IN)
#define GPIO_CIF_LV   ( 85 | GPIO_ALT_FN_3_IN)
#define GPIO_CIF_FV   ( 84 | GPIO_ALT_FN_3_IN)
#endif
